@@ -29,9 +29,9 @@ static void gx_level_slider_size_request (GtkWidget *widget, GtkRequisition *req
 static gboolean gx_level_slider_button_press (GtkWidget *widget, GdkEventButton *event);
 static gboolean gx_level_slider_pointer_motion (GtkWidget *widget, GdkEventMotion *event);
 
-G_DEFINE_TYPE(GxLEVELSlider, gx_level_slider, GX_TYPE_VSLIDER);
+G_DEFINE_TYPE(GxLevelSlider, gx_level_slider, GX_TYPE_VSLIDER);
 
-static void gx_level_slider_class_init(GxLEVELSliderClass *klass)
+static void gx_level_slider_class_init(GxLevelSliderClass *klass)
 {
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;
 
@@ -61,43 +61,11 @@ static void gx_level_slider_size_request (GtkWidget *widget, GtkRequisition *req
 	g_object_unref(pb);
 }
 
-inline double log_meter (GtkWidget *widget)
-{
-	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
-	double db = adj->value;
-	gfloat def = 0.0f; /* Meter deflection %age */
-
-	if (db < -70.0f) {
-		def = 0.0f;
-	} else if (db < -60.0f) {
-		def = (db + 70.0f) * 0.25f;
-	} else if (db < -50.0f) {
-		def = (db + 60.0f) * 0.5f + 2.5f;
-	} else if (db < -40.0f) {
-		def = (db + 50.0f) * 0.75f + 7.5f;
-	} else if (db < -30.0f) {
-		def = (db + 40.0f) * 1.5f + 15.0f;
-	} else if (db < -20.0f) {
-		def = (db + 30.0f) * 2.0f + 30.0f;
-	} else if (db < 6.0f) {
-		def = (db + 20.0f) * 2.5f + 50.0f;
-	} else {
-		def = 115.0f;
-	}
-
-	/* 115 is the deflection %age that would be
-	   when db=6.0. this is an arbitrary
-	   endpoint for our scaling.
-	*/
-
-	return def/115.0f;
-}
-
-
 static void level_slider_expose(
 	GtkWidget *widget, GdkRectangle *rect, gdouble sliderstate, GdkPixbuf *image)
 {
-	sliderstate =rect->height* log_meter (widget);
+	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
+	sliderstate = rect->height * log_meter(adj->value);
 	gdk_draw_pixbuf(GDK_DRAWABLE(widget->window), widget->style->fg_gc[0],
 	                image, 0, (gint)sliderstate, rect->x, rect->y,
 	                rect->width, rect->height, GDK_RGB_DITHER_NORMAL, 0, 0);
@@ -120,15 +88,48 @@ static gboolean gx_level_slider_expose(GtkWidget *widget, GdkEventExpose *event)
 	return FALSE;
 }
 
-static gboolean slider_set_from_pointer(GtkWidget *widget, gdouble x, gdouble y, gboolean drag, gint button, GdkEventButton *event)
+static double log_meter_inv(double def)
+{
+    def *= 115.0;
+    if (def <= 0.0) {
+        return -70.0;
+	}
+    if (def <= 2.5) {
+        return def/0.25 - 70;
+	}
+    if (def <= 7.5) {
+        return (def-2.5)/0.5 - 60;
+	}
+    if (def <= 15.0) {
+        return (def-7.5)/0.75 - 50;
+	}
+    if (def <= 30.0) {
+        return (def-15.0)/1.5 - 40;
+	}
+    if (def <= 50.0) {
+        return (def-30.0)/2.0 - 30;
+	}
+    if (def <= 115) {
+        return (def-50.0)/2.5 - 20;
+	}
+    return 6.0;
+}
+
+static inline void get_width_height(GtkWidget *widget, GdkRectangle *r)
+{
+	GdkPixbuf *pb = gtk_widget_render_icon(widget, get_stock_id(widget), GtkIconSize(-1), NULL);
+	r->width = gdk_pixbuf_get_width(pb);
+	r->height = gdk_pixbuf_get_height(pb);
+	g_object_unref(pb);
+}
+
+static gboolean slider_set_from_pointer(GtkWidget *widget, int state, gdouble x, gdouble y, gboolean drag, gint button, GdkEventButton *event)
 {
 	GdkRectangle image_rect, value_rect;
 	gint slider_height;
 	gtk_widget_style_get(widget, "slider-width", &slider_height, NULL);
-	GdkPixbuf *pb = gtk_widget_render_icon(widget, get_stock_id(widget), GtkIconSize(-1), NULL);
-	image_rect.width = gdk_pixbuf_get_width(pb);
-	image_rect.height = (gdk_pixbuf_get_height(pb) + slider_height) / 2;
-	g_object_unref(pb);
+	get_width_height(widget, &image_rect);
+	image_rect.height = (image_rect.height + slider_height) / 2;
 	x += widget->allocation.x;
 	y += widget->allocation.y;
 	_gx_regler_get_positions(GX_REGLER(widget), &image_rect, &value_rect);
@@ -148,9 +149,16 @@ static gboolean slider_set_from_pointer(GtkWidget *widget, gdouble x, gdouble y,
 	double value;
 	if (!drag) {
 		last_y = posy;
+		if (event && event->type == GDK_2BUTTON_PRESS) {
+		    gtk_range_set_value(GTK_RANGE(widget), log_meter_inv(posy/image_rect.height));
+		}
 		return TRUE;
 	}
-	value = (posy - last_y) * 0.005;
+    double sc = 0.005;
+    if (state & GDK_CONTROL_MASK) {
+        sc = 0.0005;
+    }
+	value = (posy - last_y) * sc;
 	last_y = posy;
 	gtk_range_set_value(GTK_RANGE(widget), adj->value + value * (adj->upper - adj->lower));
 	return TRUE;
@@ -163,7 +171,7 @@ static gboolean gx_level_slider_button_press (GtkWidget *widget, GdkEventButton
 		return FALSE;
 	}
 	gtk_widget_grab_focus(widget);
-	if (slider_set_from_pointer(widget, event->x, event->y, FALSE, event->button, event)) {
+	if (slider_set_from_pointer(widget, event->state, event->x, event->y, FALSE, event->button, event)) {
 		gtk_grab_add(widget);
 	}
 	return FALSE;
@@ -176,12 +184,12 @@ static gboolean gx_level_slider_pointer_motion(GtkWidget *widget, GdkEventMotion
 		return FALSE;
 	}
 	gdk_event_request_motions (event);
-	slider_set_from_pointer(widget, event->x, event->y, TRUE, 0, NULL);
+	slider_set_from_pointer(widget, event->state, event->x, event->y, TRUE, 0, NULL);
 	return FALSE;
 }
 
 
-static void gx_level_slider_init(GxLEVELSlider *level_slider)
+static void gx_level_slider_init(GxLevelSlider *level_slider)
 {
 	gtk_widget_set_name (GTK_WIDGET(level_slider),"rack_slider");
 }
