@@ -308,8 +308,6 @@ static void gx_regler_class_init(GxReglerClass *klass)
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;
 	GtkBindingSet  *binding_set;
 
-	klass->change_value_id = g_signal_lookup("change-value", GTK_TYPE_RANGE);
-
 	gobject_class->finalize = gx_regler_finalize;
 	gobject_class->set_property = gx_regler_set_property;
 	gobject_class->get_property = gx_regler_get_property;
@@ -902,7 +900,11 @@ static gchar* _gx_regler_format_value(GxRegler *regler, gdouble value)
 		return fmt;
 	} else {
 		/* insert a LRM, to prevent -20 to come out as 20- in RTL locales */
-		return g_strdup_printf ("\342\200\216%0.*f", GTK_RANGE(regler)->round_digits, value);
+		int rd = GTK_RANGE(regler)->round_digits;
+		if (rd < 0) {
+			rd = 0;
+		}
+		return g_strdup_printf ("\342\200\216%0.*f", rd, value);
 	}
 }
 
@@ -1090,7 +1092,7 @@ static void gx_regler_set_value (GtkWidget *widget, int dir_down)
 static gboolean dialog_button_press_event(
 	GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	if (event->type == GDK_BUTTON_PRESS) {
+	if (event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS) {
 		gtk_widget_destroy(GTK_WIDGET(data));
 	}
 	return TRUE;
@@ -1111,8 +1113,7 @@ static gboolean dialog_key_press_event(
 	if (event->is_modifier) {
 		return FALSE;
 	}
-	if (GTK_IS_WIDGET(data));
-		gtk_widget_destroy(GTK_WIDGET(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
 	return FALSE;
 }
 
@@ -1122,8 +1123,7 @@ static gboolean dialog_key_press_before(
 	if (event->keyval == GDK_Escape) {
 		// spinbutton to current adjustment value
 		gtk_adjustment_value_changed(GTK_SPIN_BUTTON(widget)->adjustment);
-		if (GTK_IS_WIDGET(data));
-			gtk_widget_destroy(GTK_WIDGET(data));
+		gtk_widget_destroy(GTK_WIDGET(data));
 		return TRUE;
 	}
 	return FALSE;
@@ -1132,8 +1132,7 @@ static gboolean dialog_key_press_before(
 static gboolean dialog_grab_broken(
 	GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-	if (GTK_IS_WIDGET(data));
-		gtk_widget_destroy(GTK_WIDGET(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
 	return FALSE;
 }
 
@@ -1164,34 +1163,50 @@ static gboolean map_check(
 	return FALSE;
 }
 
+static void gx_regler_relay_value(GtkAdjustment *src, GtkAdjustment *dst) {
+    gtk_adjustment_set_value(dst, gtk_adjustment_get_value(src));
+}
+
 static gboolean gx_regler_value_entry(GxRegler *regler, GdkRectangle *rect, GdkEventButton *event)
 {
+	if (event->type == GDK_2BUTTON_PRESS || event->type == GDK_3BUTTON_PRESS) {
+		return FALSE;
+	}
 	g_assert(GX_IS_REGLER(regler));
-	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(regler));
+	GtkAdjustment *dst_adj = gtk_range_get_adjustment(GTK_RANGE(regler));
 	GtkWidget *dialog = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_widget_add_events(dialog, GDK_BUTTON_PRESS_MASK|GDK_BUTTON_MOTION_MASK);
 	ensure_digits(regler);
+	GtkAdjustment *src_adj = GTK_ADJUSTMENT(
+		gtk_adjustment_new(
+			dst_adj->value, dst_adj->lower, dst_adj->upper,
+			dst_adj->step_increment, dst_adj->page_increment, dst_adj->page_size));
+	int rd = GTK_RANGE(regler)->round_digits;
+	if (rd < 0) {
+		rd = 0;
+	}
 	GtkWidget *spinner = gtk_spin_button_new(
-		GTK_ADJUSTMENT(adj), adj->step_increment,
-		GTK_RANGE(regler)->round_digits);
-	gtk_container_add (GTK_CONTAINER(dialog), spinner);
-	
+		src_adj, src_adj->step_increment, rd);
+	g_signal_connect(src_adj, "value-changed", G_CALLBACK(gx_regler_relay_value), dst_adj);
+	gtk_container_add(GTK_CONTAINER(dialog), spinner);
+
 	g_signal_connect(spinner, "button-press-event", G_CALLBACK(spinner_button_press_event), NULL);
 	g_signal_connect(dialog, "button-press-event", G_CALLBACK(dialog_button_press_event), dialog);
 	g_signal_connect(spinner, "key-press-event", G_CALLBACK(dialog_key_press_before), dialog);
 	g_signal_connect_after(spinner, "key-press-event", G_CALLBACK(dialog_key_press_event), dialog);
-	g_signal_connect_object(spinner, "activate", G_CALLBACK(gtk_widget_destroy), dialog, (GConnectFlags)(G_CONNECT_AFTER|G_CONNECT_SWAPPED));
+	g_signal_connect_object(spinner, "activate", G_CALLBACK(gtk_widget_destroy),
+				dialog, (GConnectFlags)(G_CONNECT_AFTER|G_CONNECT_SWAPPED));
 	g_signal_connect(dialog, "grab-broken-event", G_CALLBACK(dialog_grab_broken), dialog);
 	g_signal_connect(dialog, "map-event", G_CALLBACK(map_check), GTK_WIDGET(regler));
-	
-	gtk_window_move(GTK_WINDOW(dialog), -100, -100); // trick so its not visible
-	gtk_widget_show_all(dialog);
+
+	gtk_widget_show(spinner);
+	gtk_widget_realize(GTK_WIDGET(dialog));
 	GtkRequisition rq;
 	gtk_widget_get_requisition(dialog, &rq);
 	gint xorg, yorg;
 	gdk_window_get_origin(GTK_WIDGET(regler)->window, &xorg, &yorg);
-	
 	gtk_window_move(GTK_WINDOW(dialog), xorg+rect->x+(rect->width-rq.width)/2, yorg+rect->y+(rect->height-rq.height)/2);
+	gtk_widget_show(dialog);
 	return FALSE;
 }
 
@@ -1266,7 +1281,7 @@ static void gx_regler_adjustment_notified(GObject *gobject, GParamSpec *pspec)
 static void gx_regler_init(GxRegler *regler)
 {
 	regler->priv = G_TYPE_INSTANCE_GET_PRIVATE (regler, GX_TYPE_REGLER, GxReglerPrivate);
-	GTK_RANGE(regler)->round_digits = 0;
+	GTK_RANGE(regler)->round_digits = -1;
 	regler->value_position = GTK_POS_BOTTOM;
 	regler->show_value = TRUE;
 	regler->value_xalign = 0.5;
