@@ -35,11 +35,20 @@
 #include <glibmm/i18n.h>     // NOLINT
 #include <glibmm/optioncontext.h>   // NOLINT
 #include <glibmm/dispatcher.h>
+#include <glibmm/miscutils.h>
 #include <giomm/file.h>
 
 /* waf generated defines */
 #include "../config.h"
 
+// define USE_MIDI_OUT to create a midi output port and
+// make the MidiAudioBuffer plugin activatable
+// for the UI:
+// - menudef.xml: make action MidiOut visible (its commented out)
+// - ports.glade: set scrolledwindow5 and label12 to visible
+//
+//#define USE_MIDI_OUT
+
 #ifdef LADSPA_SO
 #define gettext(x) dgettext(GETTEXT_PACKAGE, x)
 #endif
