@@ -31,22 +31,6 @@ namespace gx_engine {
 
 /* -- guitarix main engine -- */
 
-/****************************************************************/
-
-class AudioVariables {
-public:
-    bool fwarn;
-    int fskin;
-
-    //FIXME:
-    float filebutton;
-
-    void register_parameter(ParamMap& pmap);
-};
-
-extern AudioVariables audio;
-
-
 /****************************************************************
  ** class ModuleSelectorFromList
  */
@@ -98,7 +82,7 @@ public:
     CabinetConvolver cabinet;
     ContrastConvolver contrast;
 public:
-    GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups);
+    GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::CmdlineOptions& options);
     ~GxEngine();
     void set_jack(gx_jack::GxJack *jack) { midiaudiobuffer.set_jack(jack); }
 };
