@@ -23,7 +23,7 @@
 
 #include "gx_plugin.h"
 
-#define declare(n) namespace n { extern PluginDef plugin; }
+#define declare(n) namespace n { PluginDef *plugin(); }
 
 namespace gx_engine {
 
@@ -45,6 +45,7 @@ declare(gxamp7)
 declare(gxamp8)
 declare(gxamp16)
 declare(gxamp6)
+declare(gxamp18)
 declare(gx_ampmodul)
 }
 
