@@ -66,21 +66,24 @@ public:
  ** class TunerAdapter
  */
 
-class TunerAdapter: public ModuleSelector, private PluginDef {
+class TunerAdapter: public ModuleSelector, private PluginDef, public sigc::trackable {
 private:
     static void feed_tuner(int count, float *input, float *output, PluginDef*);
     static int regparam(const ParamReg& reg);
+    static int activate(bool start, PluginDef *plugin);
     static void init(unsigned int samplingFreq, PluginDef *plugin);
     PitchTracker pitch_tracker;
     int state;
     ModuleSequencer& engine;
-    enum { tuner_use = 0x01, midi_use = 0x02 };
+    enum { tuner_use = 0x01, livetuner_use = 0x02, switcher_use = 0x04, midi_use = 0x08 };
     void set_and_check(int use, bool on);
     Plugin* dep_plugin;
 public:
     Plugin plugin;
     TunerAdapter(ModuleSequencer& engine);
+    void used_for_switching(bool on) { set_and_check(switcher_use, on); }
     void used_for_display(bool on) { set_and_check(tuner_use, on); }
+    void used_for_livedisplay(bool on) { set_and_check(livetuner_use, on); }
     void used_by_midi(bool on) { set_and_check(midi_use, on); }
     void set_dep_module(Plugin* dep) { dep_plugin = dep; }
     void set_module();
@@ -164,23 +167,30 @@ public:
     void init(int samplingFreq);
     void process_midi(int len, float *audiodata, void *midi_port_buf, float jcpu_load,
 		      float fConsta4, float fConsta1t);
-    bool *get_midistat_pointer() { return &midistat; }
+    bool get_midistat() { return midistat; }
 };
 
 
 class MidiAudioBuffer: PluginDef {
+public:
+    enum Load { load_off = -1, load_normal = 0, load_over = 1 };
 private:
     MidiVariables midi;
     gx_engine::TunerAdapter& tuner;
     gx_jack::GxJack* jack;
+    Load jack_overload;
+    Glib::Dispatcher overload_change;
     static void fill_buffer(int count, float *input0, float *output0, PluginDef*);
     static int regparam(const ParamReg& reg);
     static void init(unsigned int samplingFreq, PluginDef *plugin);
+    static int activate(bool start, PluginDef *plugin);
 public:
     Plugin plugin;
     MidiAudioBuffer(TunerAdapter& t);
     void set_jack(gx_jack::GxJack* jack_) { jack = jack_; }
-    bool *get_midistat_pointer() { return midi.get_midistat_pointer(); }
+    bool get_midistat() { return midi.get_midistat(); }
+    Load jack_load_status() { return jack_overload; }
+    Glib::Dispatcher& signal_jack_load_change() { return overload_change; }
 };
 
 
@@ -232,11 +242,11 @@ public:
  ** class GxJConvSettings
  */
 
-class GxJConvSettings: boost::noncopyable {
+class GxJConvSettings {
  private:
     // main jconv setting
-    string          fIRFile;
-    string          fIRDir;
+    std::string     fIRFile;
+    std::string     fIRDir;
 
     float           fGain;       // jconv gain
     guint           fOffset;     // offset in IR where to start comvolution
@@ -244,28 +254,26 @@ class GxJConvSettings: boost::noncopyable {
     guint           fDelay;      // delay when to apply reverb
     Gainline        gainline;
     bool            fGainCor;
-    sigc::signal<void> file_changed;
     void read_gainline(gx_system::JsonParser& jp);
-    void read_favorites(gx_system::JsonParser& jp);
-    inline void setIRFile(string name)            { fIRFile = name; }
     inline void setIRDir(string name)             { fIRDir = name; }
-
+    friend class ConvolverAdapter;
+    friend class ParameterV<GxJConvSettings>;
  public:
     GxJConvSettings();
     GxJConvSettings& operator=(GxJConvSettings const& jcset);
-
+    bool operator==(const GxJConvSettings& jcset) const;
     // getters and setters
-    inline const string& getIRFile() const               { return fIRFile; }
-    string getFullIRPath() const;
+    inline const std::string& getIRFile() const   { return fIRFile; }
+    std::string getFullIRPath() const;
     inline float           getGain() const        { return fGain; }
     inline guint           getOffset() const      { return fOffset; }
     inline guint           getLength() const      { return fLength; }
     inline guint           getDelay() const       { return fDelay; }
     inline bool            getGainCor() const     { return fGainCor; }
     inline const Gainline& getGainline() const    { return gainline; }
-    inline const string& getIRDir() const         { return fIRDir; }
+    const std::string& getIRDir() const           { return fIRDir; }
     void setFullIRPath(string name);
-
+    inline void setIRFile(string name)            { fIRFile = name; }
     inline void setGain(float gain)               { fGain       = gain; }
     inline void setGainCor(bool gain)             { fGainCor    = gain; }
     inline void setOffset(guint offs)             { fOffset     = offs; }
@@ -274,18 +282,40 @@ class GxJConvSettings: boost::noncopyable {
     inline void setGainline(const Gainline& gain) { gainline    = gain; }
 
  public:
-
-    // checkbutton state
-    static bool* checkbutton7;
-    list<Glib::ustring> faflist;
-    typedef list<Glib::ustring>::iterator faf_iterator;
     void readJSON(gx_system::JsonParser& jp,
 		  const gx_system::PathList& search_path);
     void writeJSON(gx_system::JsonWriter& w,
 		   const gx_system::PathList& search_path);
-    inline sigc::signal<void>& signal_file_changed() { return file_changed; }
 };
 
+class ConvolverAdapter;
+
+template<>
+class ParameterV<GxJConvSettings>: public Parameter {
+private:
+    GxJConvSettings json_value;
+    GxJConvSettings *value;
+    GxJConvSettings std_value;
+    ConvolverAdapter &conv;
+public:
+    ParameterV(const string& id, ConvolverAdapter &conv_, GxJConvSettings *v);
+    ~ParameterV();
+    static ParameterV<GxJConvSettings> *insert_param(
+	ParamMap &pmap, const string& id, ConvolverAdapter &conv, GxJConvSettings *v);
+    bool set(const GxJConvSettings& val) const;
+    const GxJConvSettings& get_value() const { return *value; }
+    virtual void *zone();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
+    virtual void set(float n, float high, float llimit, float ulimit);
+    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual bool compareJSON_value();
+    virtual void setJSON_value();
+    virtual void readJSON_value(gx_system::JsonParser& jp);
+};
+
+typedef ParameterV<GxJConvSettings> JConvParameter;
+
 
 /****************************************************************
  ** class ConvolverAdapter
@@ -300,6 +330,10 @@ private:
     GxConvolver conv;
     boost::mutex activate_mutex;
     EngineControl& engine;
+    sigc::slot<void> sync;
+    ParamMap& param;
+    const gx_system::PathList& pathlist;
+    std::string sys_ir_dir;
     bool activated;
     jconv_post::Dsp jc_post;
     // wrapper for the rack order function pointers
@@ -309,17 +343,27 @@ private:
     static int convolver_register(const ParamReg& reg);
     static void convolver_init(unsigned int samplingFreq, PluginDef *pdef);
     void change_buffersize(unsigned int size);
+    sigc::signal<void> settings_changed;
+    GxJConvSettings jcset;
+    JConvParameter *jcp;
 public:
     Plugin plugin;
-    GxJConvSettings jcset;
 public:
-    ConvolverAdapter(EngineControl& engine);
+    ConvolverAdapter(EngineControl& engine, sigc::slot<void> sync, ParamMap& param,
+		     const gx_system::PathList& pathlist, const std::string& sys_ir_dir);
     ~ConvolverAdapter();
     void restart();
     bool conv_start();
-    inline sigc::signal<void>& signal_file_changed() { return jcset.signal_file_changed(); }
-    inline const string& getIRFile() const { return jcset.getIRFile(); }
+    inline sigc::signal<void>& signal_settings_changed() { return settings_changed; }
+    inline const std::string& getIRFile() const { return jcset.getIRFile(); }
     inline void set_sync(bool val) { conv.set_sync(val); }
+    inline std::string getFullIRPath() const { return jcset.getFullIRPath(); }
+    inline const std::string& getIRDir() const { return jcset.getIRDir(); }
+    bool set(const GxJConvSettings& jcset) const { return jcp->set(jcset); }
+    const GxJConvSettings& get_jcset() const { return jcset; }
+    const gx_system::PathList& get_pathlist() const { return pathlist; }
+    const std::string& get_sys_IR_dir() const { return sys_ir_dir; }
+    ParamMap& get_parameter_map() const { return param; }
 };
 
 
@@ -333,20 +377,21 @@ protected:
     GxSimpleConvolver conv;
     boost::mutex activate_mutex;
     EngineControl& engine;
+    sigc::slot<void> sync;
     bool activated;
+    sigc::connection update_conn;
     static void init(unsigned int samplingFreq, PluginDef *p);
     static int activate(bool start, PluginDef *pdef);
     void change_buffersize(unsigned int);
-    inline bool is_runnable() { return conv.is_runnable(); }
-    inline void set_not_runnable() { conv.set_not_runnable(); }
     int conv_start();
+    bool check_update_timeout();
+    virtual void check_update() = 0;
+    virtual bool start(bool force = false) = 0;
 public:
     Plugin plugin;
 public:
-    BaseConvolver(EngineControl& engine, gx_resample::BufferResampler& resamp);
+    BaseConvolver(EngineControl& engine, sigc::slot<void> sync, gx_resample::BufferResampler& resamp);
     virtual ~BaseConvolver();
-    virtual bool start(bool force = false) = 0;
-    inline void conv_stop() { conv.stop(); }
     inline void set_sync(bool val) { conv.set_sync(val); }
 };
 
@@ -364,19 +409,18 @@ private:
     float sum;
     value_pair *cab_names;
     cabinet_impulse_former::Dsp impf;
-    inline void compensate_cab(int count, float *input0, float *output0);
     static void run_cab_conf(int count, float *input, float *output, PluginDef*);
     static int register_cab(const ParamReg& reg);
-    bool update();
-public:
-    CabinetConvolver(EngineControl& engine, gx_resample::BufferResampler& resamp);
-    ~CabinetConvolver();
-    bool start(bool force = false);
+    bool do_update();
+    virtual void check_update();
+    virtual bool start(bool force = false);
     bool cabinet_changed() { return current_cab != cabinet; }
     void update_cabinet() { current_cab = cabinet; }
     bool sum_changed() { return abs(sum - (level + bass + treble)) > 0.01; }
     void update_sum() { sum = level + bass + treble; }
-    bool conv_update();
+public:
+    CabinetConvolver(EngineControl& engine, sigc::slot<void> sync, gx_resample::BufferResampler& resamp);
+    ~CabinetConvolver();
 };
 
 
@@ -389,16 +433,16 @@ private:
     float level;
     float sum;
     presence_level::Dsp presl;
-    // wrapper for the rack order function pointers
-    inline void compensate_con(int count, float *input0, float *output0);
     static void run_contrast(int count, float *input, float *output, PluginDef*);
     static int register_con(const ParamReg& reg);
     inline void update_sum() { sum = level; }
-public:
-    ContrastConvolver(EngineControl& engine, gx_resample::BufferResampler& resamp);
-    ~ContrastConvolver();
+    virtual void check_update();
+    bool do_update();
     inline bool sum_changed() { return abs(sum - level) > 0.01; }
-    bool start(bool force = false);
+    virtual bool start(bool force = false);
+public:
+    ContrastConvolver(EngineControl& engine, sigc::slot<void> sync, gx_resample::BufferResampler& resamp);
+    ~ContrastConvolver();
 };
 
 } // namespace gx_engine
