@@ -52,7 +52,7 @@ class MidiControllerTable: public sigc::trackable {
     GtkToggleButton *togglebutton;
     GtkTreeSelection *selection;
     GtkListStore *store;
-    Gtk::CheckMenuItem& menuitem;
+    Glib::RefPtr<Gtk::ToggleAction> menuaction;
     gx_engine::ParamMap& param;
     static void response_cb(GtkWidget *widget, gint response_id, gpointer data);
     static void edited_cb(GtkCellRendererText *renderer, gchar *path,
@@ -61,10 +61,10 @@ class MidiControllerTable: public sigc::trackable {
     static void toggleButtonSetSwitch(GtkWidget *w, gpointer data);
     void set(bool v);
     void load();
-    explicit MidiControllerTable(gx_engine::ParamMap& param, Gtk::CheckMenuItem& item);
+    explicit MidiControllerTable(gx_engine::ParamMap& param, Glib::RefPtr<Gtk::ToggleAction> item);
     ~MidiControllerTable();
  public:
-    static void toggle(gx_engine::ParamMap& param, Gtk::CheckMenuItem& item);
+    static void toggle(gx_engine::ParamMap& param, Glib::RefPtr<Gtk::ToggleAction> item);
 };
 
 /*****************************************************************
@@ -75,12 +75,14 @@ class MidiConnect {
  private:
     enum { RESPONSE_DELETE = 1 };
     gx_engine::Parameter &param;
+    int current_control;
     GtkAdjustment* adj_lower;
     GtkAdjustment* adj_upper;
+    GtkToggleButton *use_toggle;
+    //
     GtkWidget* dialog;
     GtkWidget* entry_new;
     GtkWidget* label_desc;
-    int current_control;
     static string ctr_desc(int ctr);
     static const char *ctl_to_str(int n);
  public:
