@@ -55,7 +55,6 @@ protected:
     int steps_up;		// RT; >= 1
     int steps_up_dead;		// RT; >= 0
     int steps_down;		// RT; >= 1
-    volatile bool latch; // RT; set between commit and end of rt cycle
     list<Plugin*> modules;
     inline void set_ramp_value(int n) { gx_system::atomic_set(&ramp_value, n); } // RT
     inline void set_ramp_mode(RampMode n) { gx_system::atomic_set(&ramp_mode, n); } // RT
@@ -72,13 +71,15 @@ public:
     void clear_module_states();
     inline void post_rt_finished() { // RT
 	int val;
-	latch = false;
-	if (sem_getvalue(&sync_sem, &val) == 0 && val == 0) {
+	sem_getvalue(&sync_sem, &val);
+	if (val == 0) {
 	    sem_post(&sync_sem);
 	}
     }
     bool wait_rt_finished();
-    void wait_latch();
+    void set_latch();
+    void wait_latch() { wait_rt_finished(); }
+    void sync() { set_latch(); wait_latch(); }
     inline bool check_release() { return !to_release.empty(); }
     void release();
     void wait_ramp_down_finished();
@@ -191,7 +192,10 @@ void ThreadSafeChainPointer<F>::commit(bool clear) {
     for (list<Plugin*>::const_iterator p = modules.begin(); p != modules.end(); p++) {
 	PluginDef* pd = (*p)->pdef;
 	if (pd->activate_plugin) {
-	    pd->activate_plugin(true, pd);
+	    if (pd->activate_plugin(true, pd) != 0) {
+		(*p)->on_off = false;
+		continue;
+	    }
 	} else if (pd->clear_state && clear) {
 	    pd->clear_state(pd);
 	}
@@ -201,7 +205,7 @@ void ThreadSafeChainPointer<F>::commit(bool clear) {
     }
     current_pointer[active_counter].func = 0;
     gx_system::atomic_set(&processing_pointer, current_pointer);
-    latch = true;
+    set_latch();
     current_index = (current_index+1) % 2;
     current_pointer = rack_order_ptr[current_index];
 }
@@ -273,7 +277,7 @@ public:
  */
 
 enum GxEngineState {  // engine states set by user (ModuleSequencer set_state/get_state)
-    kEngineOff    = 0,  // mute, no output (by tuner or something might run)
+    kEngineOff    = 0,  // mute, no output (but tuner or something might run)
     kEngineOn     = 1,  // normal operation
     kEngineBypass = 2   // just some balance or level control
 };
@@ -284,6 +288,7 @@ protected:
     int audio_mode;     // GxEngineState coded as PGN_MODE_XX flags
     boost::mutex stateflags_mutex;
     int stateflags;
+    sigc::signal<void, GxEngineState> state_change;
 public:
     MonoModuleChain mono_chain;  // active modules (amp chain, input to insert output)
     StereoModuleChain stereo_chain;  // active stereo modules (effect chain, after insert input)
@@ -329,6 +334,7 @@ public:
     void clear_stateflag(StateFlag flag);
     void set_state(GxEngineState state);
     GxEngineState get_state();
+    sigc::signal<void, GxEngineState>& signal_state_change() { return state_change; }
 #ifndef NDEBUG
     void print_engine_state();
 #endif
