@@ -87,7 +87,7 @@ typedef ParameterV<float> FloatParameter;
 typedef ParameterV<int> IntParameter;
 typedef ParameterV<unsigned int> UIntParameter;
 typedef ParameterV<bool> BoolParameter;
-typedef ParameterV<string> StringParameter;
+typedef ParameterV<Glib::ustring> StringParameter;
 
 class FloatEnumParameter;
 class EnumParameter;
@@ -101,7 +101,7 @@ class Parameter: boost::noncopyable {
 public:
     enum ctrl_type { None, Continuous, Switch, Enum };
 protected:
-    enum value_type { tp_float, tp_int, tp_uint, tp_bool, tp_switch, tp_file, tp_string };
+    enum value_type { tp_float, tp_int, tp_uint, tp_bool, tp_switch, tp_file, tp_string, tp_special };
     string _id;
     string _name, _group, _desc;
     enum value_type v_type : 3;
@@ -109,8 +109,10 @@ protected:
     bool save_in_preset : 1;
     bool controllable : 1;
     bool own_var : 1;
+    bool do_not_save : 1;
     bool used : 1; // debug
-
+protected:
+    void range_warning(float value, float lower, float upper);
 public:
     Parameter(const string& id, const string& name, value_type vtp, ctrl_type ctp, bool preset,
               bool ctrl):
@@ -122,6 +124,7 @@ public:
         save_in_preset(preset),
         controllable(ctrl),
 	own_var(false),
+	do_not_save(false),
         used(false) {}
     virtual ~Parameter();
 
@@ -143,6 +146,8 @@ public:
     ctrl_type getControlType() const { return c_type; }
     bool isControllable() const { return controllable; }
     bool isInPreset() const { return save_in_preset; }
+    bool isSavable() const { return !do_not_save; }
+    void setSavable(bool v) { do_not_save = !v; }
     const string& id() const { return _id; }
     const string& group() const { return _group; }
     string l_group() const { return gettext(_group.c_str()); }
@@ -153,11 +158,13 @@ public:
     string l_desc() const { return gettext(_desc.c_str()); }
     bool operator==(const Parameter& p) const { return &p == this; }
     virtual void *zone() = 0;
-    virtual void set_std_value() = 0;
+    virtual void stdJSON_value() = 0;
     virtual void set(float n, float high, float llimit, float ulimit) = 0;
+    virtual bool on_off_value() = 0;
     virtual void writeJSON(gx_system::JsonWriter& jw) = 0;
     virtual void readJSON_value(gx_system::JsonParser& jp) = 0;
     virtual void setJSON_value() = 0;
+    virtual bool compareJSON_value() = 0;
     virtual bool hasRange() const;
     virtual float getLowerAsFloat() const;
     virtual float getUpperAsFloat() const;
@@ -204,10 +211,12 @@ public:
     float &get_value() const { return *value; }
     void convert_from_range(float low, float up);
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
     virtual void readJSON_value(gx_system::JsonParser& jp);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual bool hasRange() const;
     virtual float getLowerAsFloat() const;
@@ -218,6 +227,7 @@ public:
 	Parameter(id, name, tp_float, ctp, preset, ctrl),
 	value(v ? v : new float()), std_value(sv),lower(lv),upper(uv),step(tv) {
 	own_var = !v;
+	*value = sv;
     }
 #ifndef NDEBUG
     friend void compare_parameter(const char* title, Parameter* p1,
@@ -253,10 +263,12 @@ public:
     void set(int val) const { *value = min(max(val, lower), upper); }
     int &get_value() const { return *value; }
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
     virtual void readJSON_value(gx_system::JsonParser& jp);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual bool hasRange() const;
     virtual float getLowerAsFloat() const;
@@ -267,6 +279,7 @@ public:
 	Parameter(id, name, tp_int, ctp, preset, ctrl),
         value(v ? v : new int()), std_value(sv), lower(lv), upper(uv) {
 	own_var = !v;
+	*value = sv;
     }
     ~ParameterV();
 };
@@ -299,10 +312,12 @@ public:
     void set(unsigned int val) const { *value = min(max(val, lower), upper); }
     unsigned int &get_value() const { return *value; }
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
     virtual void readJSON_value(gx_system::JsonParser& jp);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual bool hasRange() const;
     virtual float getLowerAsFloat() const;
@@ -313,6 +328,7 @@ public:
         Parameter(id, name, tp_uint, ctp, preset, ctrl),
         value(v ? v : new unsigned int()), std_value(sv), lower(lv), upper(uv) {
 	own_var = !v;
+	*value = sv;
     }
     ~ParameterV();
 };
@@ -343,10 +359,12 @@ public:
     void set(bool val) const { *value = val; }
 public:
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
     bool &get_value() const { return *value; }
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual void readJSON_value(gx_system::JsonParser& jp);
     ParameterV(const string& id, const string& name, ctrl_type ctp, bool preset,
@@ -354,6 +372,7 @@ public:
         Parameter(id, name, tp_bool, ctp, preset, ctrl),
         value(v ? v : new bool()), std_value(sv) {
 	own_var = !v;
+	*value = sv;
     }
     ~ParameterV();
 };
@@ -369,12 +388,14 @@ private:
 public:
     sigc::signal<void, bool>& signal_changed() { return changed; }
     void set(bool val);
-    bool get() const { return value; }
+    const bool& get() const { return value; }
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
     virtual void readJSON_value(gx_system::JsonParser& jp);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     SwitchParameter(const string& id, bool preset = false, bool sv = false):
         Parameter(id, "", tp_switch, Switch, preset, false),
@@ -395,10 +416,12 @@ public:
     void set_path(const string& path);
     const Glib::RefPtr<Gio::File>& get() const { return value; }
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
     virtual void readJSON_value(gx_system::JsonParser& jp);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     FileParameter(const string& id, const string& filename, bool preset = false):
         Parameter(id, "", tp_file, None, preset, false),
@@ -421,29 +444,32 @@ public:
 /****************************************************************/
 
 template<>
-class ParameterV<string>: public Parameter {
+class ParameterV<Glib::ustring>: public Parameter {
 private:
-    string json_value;
-    string *value;
+    Glib::ustring json_value;
+    Glib::ustring *value;
 public:
-    string std_value;
+    Glib::ustring std_value;
 public:
-    void set(const string& val) const { *value = val; }
-    string &get_value() const { return *value; }
+    void set(const Glib::ustring& val) const { *value = val; }
+    Glib::ustring &get_value() const { return *value; }
     virtual void *zone();
-    virtual void set_std_value();
+    virtual void stdJSON_value();
+    virtual bool on_off_value();
     virtual void set(float n, float high, float llimit, float ulimit);
     virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual void readJSON_value(gx_system::JsonParser& jp);
-    ParameterV(const string& id, const string& name, string *v, const string& sv)
-	: Parameter(id, name, tp_string, None, false, false),
-	  value(v ? v : new string), std_value(sv) {
+    ParameterV(const string& id, const string& name, Glib::ustring *v, const Glib::ustring& sv, bool preset = false)
+	: Parameter(id, name, tp_string, None, preset, false),
+	  value(v ? v : new Glib::ustring), std_value(sv) {
 	own_var = !v;
     }
     ~ParameterV();
 };
 
+
 /****************************************************************/
 
 inline FloatParameter &Parameter::getFloat() {
@@ -511,6 +537,7 @@ class ParamMap: boost::noncopyable {
     void insert(Parameter* param); // private so we can make sure parameters are owned
 
  public:
+    template<class T> friend class ParameterV;
     ParamMap();
     ~ParamMap();
     typedef map<string, Parameter*>::const_iterator iterator;
@@ -532,6 +559,8 @@ class ParamMap: boost::noncopyable {
         return *id_map[p];
     }
     void set_init_values();
+    void reset_unit(Glib::ustring group_id) const;
+    bool unit_has_std_values(Glib::ustring group_id) const;
     inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std,
 				   float lower, float upper, float step) {
 	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, step, true);
@@ -549,8 +578,8 @@ class ParamMap: boost::noncopyable {
 	insert(p);
 	return p;
     }
-    inline BoolParameter *reg_par(const string& id, const string& name, bool *var, bool std = false) {
-	BoolParameter * p = new BoolParameter(id, name, Parameter::Switch, true, var, std, true);
+    inline BoolParameter *reg_par(const string& id, const string& name, bool *var, bool std=false, bool preset=true) {
+	BoolParameter * p = new BoolParameter(id, name, Parameter::Switch, preset, var, std, true);
 	insert(p);
 	return p;
     }
@@ -581,7 +610,7 @@ class ParamMap: boost::noncopyable {
 	return p;
     }
     inline BoolParameter *reg_non_midi_par(const string& id, bool *var, bool preset, bool std = false) {
-	BoolParameter *p = new BoolParameter(id, "", Parameter::None, preset, var, std, false);
+	BoolParameter *p = new BoolParameter(id, "", Parameter::Switch, preset, var, std, false);
 	insert(p);
 	return p;
     }
@@ -606,8 +635,8 @@ class ParamMap: boost::noncopyable {
 	insert(p);
 	return p;
     }
-    inline StringParameter *reg_string(const string& id, const string& name, string *var, const string& sv) {
-	StringParameter *p = new StringParameter(id, name, var, sv);
+    inline StringParameter *reg_string(const string& id, const string& name, Glib::ustring *var, const string& sv, bool preset=false) {
+	StringParameter *p = new StringParameter(id, name, var, sv, preset);
 	insert(p);
 	return p;
     }
@@ -663,15 +692,17 @@ class MidiController {
  private:
     Parameter *param;
     float _lower, _upper;
+    bool toggle;
  public:
-    MidiController(Parameter& p, float l, float u):
-        param(&p), _lower(l), _upper(u) {}
+    MidiController(Parameter& p, float l, float u, bool t=false):
+        param(&p), _lower(l), _upper(u), toggle(t) {}
     float lower() const { return _lower; }
     float upper() const { return _upper; }
+    bool is_toggle() const { return toggle; }
     bool hasParameter(const Parameter& p) const { return *param == p; }
     Parameter& getParameter() const { return *param; }
     static MidiController* readJSON(gx_system::JsonParser&, ParamMap& param);
-    void set(int n) { param->set(n, 127, _lower, _upper); }
+    void set_midi(int n, int last_value);
     void set(float v, float high) { param->set(v, high, _lower, _upper); }
     void writeJSON(gx_system::JsonWriter& jw) const;
 };
@@ -689,11 +720,10 @@ class MidiControllerList {
     enum { controller_array_size = 128 };
  private:
     controller_array       map;
+    static int             last_midi_control_value[controller_array_size];
     bool                   midi_config_mode;
     int                    last_midi_control;
-    int                    last_midi_control_value;
     volatile gint          program_change;
-    sem_t                  program_change_sem;
     Glib::Dispatcher       pgm_chg;
     sigc::signal<void>     changed;
     sigc::signal<void,int> new_program;
@@ -708,17 +738,22 @@ class MidiControllerList {
     void set_current_control(int ctl) { last_midi_control = ctl; }
     void set_ctr_val(int ctr, int val);
     void deleteParameter(Parameter& param, bool quiet = false);
-    void modifyCurrent(Parameter& param, float lower, float upper);
+    void modifyCurrent(Parameter& param, float lower, float upper, bool toggle);
     int param2controller(Parameter& param, const MidiController** p);
     void writeJSON(gx_system::JsonWriter& jw);
     static void readJSON(gx_system::JsonParser& jp, ParamMap& param, controller_array& m);
     static controller_array* create_controller_array() {
 	return new controller_array(controller_array_size); }
+    static int get_last_midi_control_value(unsigned int n) { assert(n < controller_array_size); return last_midi_control_value[n]; }
+    static void set_last_midi_control_value(unsigned int n, int v) { assert(n < controller_array_size); last_midi_control_value[n] = v; }
+    static void *get_midi_control_zone(unsigned int n) { assert(n < controller_array_size); return &last_midi_control_value[n]; }
     void set_controller_array(const controller_array& m);
     void remove_controlled_parameters(paramlist& plist, const controller_array *m);
     sigc::signal<void>& signal_changed() { return changed; }
     sigc::signal<void,int>& signal_new_program() { return new_program; }
     void compute_midi_in(void* midi_input_port_buf);
+    void update_from_controller(int ctr);
+    void update_from_controllers();
 };
 
 extern MidiControllerList controller_map; // map ctrl num -> controlled parameters
