@@ -33,8 +33,9 @@ namespace gx_engine {
 */
 
 enum {			       // additional flags for PluginDef (used internally)
-    PGNI_DYN_POSITION = 0x1000, // plugin is part of dynamically ordered rack
-    PGNI_NOT_OWN      = 0x2000,	// not owned by PluginList
+    PGNI_DYN_POSITION = 0x10000, // plugin is part of dynamically ordered rack
+    PGNI_NOT_OWN      = 0x20000, // not owned by PluginList
+    PGNI_UI_REG       = 0x40000, // Plugin registered in user interface
 };
 
 class Plugin {
@@ -56,7 +57,7 @@ public:
 
 class UiBuilderBase: public UiBuilder {
 public:
-    virtual void load(Plugin *p) = 0;
+    virtual bool load(Plugin *p) = 0;
 };
 
 /****************************************************************
@@ -88,13 +89,12 @@ private:
     gx_ui::GxUI& ui;
     list<gx_ui::GxUiItem*> rackchanger;
     int plugin_pos[PLUGIN_POS_COUNT];
-    Plugin *find_plugin(const char *id);
+    Plugin *find_plugin(const char *id) const;
     int add_module(Plugin *pl, PluginPos pos, int flags);
 public:
-    class iterator;
     PluginList(gx_ui::GxUI& ui, EngineControl& seq);
     ~PluginList();
-    Plugin *lookup_plugin(const char *id);
+    Plugin *lookup_plugin(const char *id) const;
     void set_samplerate(int samplerate); // call set_samplerate of all plugins
     int* pos_var(const char *id);     // return the position of the plugin
     bool* on_off_var(const char *id); // return the on/off switch variable of the plugin
