@@ -25,6 +25,8 @@
 #ifndef SRC_HEADERS_GX_PRESET_H_
 #define SRC_HEADERS_GX_PRESET_H_
 
+class PosixSignals;
+
 namespace gx_preset {
 
 /****************************************************************
@@ -39,7 +41,7 @@ private:
     const gx_system::CmdlineOptions& opt;
     gx_engine::paramlist plist;
     gx_engine::MidiControllerList::controller_array *m;
-    gx_engine::GxJConvSettings jcset;
+    gx_engine::GxJConvSettings *jcset;
     void read_parameters(gx_system::JsonParser &jp, bool preset);
     void write_parameters(gx_system::JsonWriter &w, bool preset);
     void clear();
@@ -72,45 +74,55 @@ public:
     void write_state(gx_system::JsonWriter &jw, bool preserve_preset);
 };
 
+class PluginPresetList: public Glib::Object {
+private:
+    std::string filename;
+    gx_engine::ParamMap& pmap;
+    ifstream is;
+    gx_system::JsonParser jp;
+    PluginPresetList(const std::string& fname, gx_engine::ParamMap& pmap);
+public:
+    static Glib::RefPtr<PluginPresetList> create(const std::string& fname, gx_engine::ParamMap& pmap);
+    bool start();
+    bool next(Glib::ustring& name, bool *is_set = 0);
+    void set(const Glib::ustring& name);
+    void write_values(gx_system::JsonWriter& jw, std::string id);
+    void save(const Glib::ustring& name, const std::string& id);
+    void remove(const Glib::ustring& name);
+};
+
 class GxSettings: public sigc::trackable, public gx_system::GxSettingsBase {
 private:
+    gx_engine::ParamMap&  param;
     gx_preset::PresetIO   preset_io;
     gx_preset::StateIO    state_io;
-    gx_engine::FileParameter& presetfile_parameter;
     bool                  state_loaded;
     bool                  no_autosave;
     gx_jack::GxJack&      jack;
     gx_system::CmdlineOptions& options;
     gx_engine::StringParameter& preset_parameter;
-    gx_engine::StringParameter& factory_parameter;
-    static GxSettings *instance;//FIXME
-    void presetfile_changed();
+    gx_engine::StringParameter& bank_parameter;
     void exit_handler(bool otherthread);
     void jack_client_changed();
     string make_state_filename();
     string make_default_state_filename();
-    string make_std_preset_filename();
-    void check_convert_presetfile();
-    void parse_factory_list();
-    static string get_default_presetfile(gx_system::CmdlineOptions& opt);
+    static bool check_create_config_dir(const Glib::ustring& dir);
+    static GxSettings *instance;
+    friend class ::PosixSignals;
 public:
+    using GxSettingsBase::banks;
     GxSettings(gx_system::CmdlineOptions& opt, gx_jack::GxJack& jack, gx_engine::ConvolverAdapter& cvr,
 	       gx_engine::MidiStandardControllers& mstdctr, gx_engine::MidiControllerList& mctrl,
 	       gx_engine::ModuleSequencer& seq, gx_engine::ParamMap& param);
     ~GxSettings();
-    string get_displayname();
-    static void check_settings_dir(gx_system::CmdlineOptions& opt);
-    void load(Source src, const string& name = "", const string& factory = "");
+    inline gx_engine::ParamMap&  get_param() const { return param; }
+    inline gx_system::CmdlineOptions& get_options() const { return options; }
+    static bool check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_new_preset);
     void loadstate();
     void disable_autosave(bool v) { no_autosave = v; }
     void auto_save_state();
-    static GxSettings& get_instance() { assert(instance); return *instance; }
-    void set_std_presetfile() { presetfile_parameter.set_std_value(); }
-    string get_preset_filename() { return presetfile_parameter.get_path(); }
-    bool set_preset_file(const string& newfile);
-    string get_preset_dirname() { return presetfile_parameter.get_directory_path(); }
-    void copy_preset_file(const string& destination) { presetfile_parameter.copy(destination); }
-    sigc::signal<void>& signal_presetfile_changed() { return presetfile_parameter.signal_changed(); }
+    Glib::RefPtr<PluginPresetList> load_plugin_preset_list(const Glib::ustring& id, bool factory) const;
+    void create_default_scratch_preset();
 };
 
 /* --------------------------------------------------------------------- */
