@@ -25,6 +25,51 @@
 #ifndef SRC_HEADERS_GX_SYSTEM_H_
 #define SRC_HEADERS_GX_SYSTEM_H_
 
+#ifndef NDEBUG
+#include <fenv.h>
+
+#ifdef __i386__
+#define FE_DENORM __FE_DENORM
+inline void clear_fpu_status_bits() { __asm__ ("fnclex"); }
+inline unsigned int get_fpu_status_bits() {
+    unsigned int fpu_status __attribute__ ((__mode__ (__HI__)));
+    __asm__("fnstsw %0" : "=m" (*&fpu_status));
+	return fpu_status;
+}
+#else
+inline void clear_fpu_status_bits() { feclearexcept(FE_ALL_EXCEPT); }
+inline unsigned int get_fpu_status_bits() {
+    fexcept_t flagp;
+    int ret = fegetexceptflag(&flagp, FE_ALL_EXCEPT);
+    assert(ret == 0);
+    return flagp;
+}
+#endif //__i386__
+#endif // !NDEBUG
+#ifdef __SSE__
+
+/* On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
+   flags to avoid costly denormals */
+#ifdef __SSE3__
+#include <pmmintrin.h>
+inline void AVOIDDENORMALS() {
+    _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
+    _MM_SET_DENORMALS_ZERO_MODE(_MM_DENORMALS_ZERO_ON);
+}
+#else
+#include <xmmintrin.h>
+inline void AVOIDDENORMALS() { _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON); }
+#endif //__SSE3__
+
+#else
+
+inline void _MM_SET_EXCEPTION_STATE(unsigned int __mask) {}
+inline unsigned int _MM_GET_EXCEPTION_STATE(void) { return 0; }
+inline void AVOIDDENORMALS() {}
+
+#endif //__SSE__
+
+
 /* constant defines */
 #define ASCII_START (48)
 #define GDK_NO_MOD_MASK (GdkModifierType)0
@@ -109,77 +154,82 @@ inline void Accum::add(int diff) {
 }
 
 
-class Measure {
- private:
+struct Measure {
     Accum period;
     Accum duration;
     Accum duration1;
     Accum duration2;
-    timespec t1;
-    timespec t2;
-    static int ts_diff(struct timespec ts1, struct timespec ts2);
+    unsigned int FPUStatus1;
+    unsigned int MXStatus1;
+    unsigned int FPUStatus2;
+    unsigned int MXStatus2;
     inline float ns2ms(int n) const { return n * 1e-6; }
-
- public:
-    inline void reset() {
-        period.reset();
-        duration.reset();
-        duration1.reset();
-        duration2.reset();
-        t1.tv_sec = 0;
-        t1.tv_nsec = 0;
-        t2.tv_sec = 0;
-        t2.tv_nsec = 0;
-    }
+    void reset();
     Measure() { reset(); }
-    void start_process();
-    void pause_process();
-    void cont_process();
-    void stop_process();
     void print_accum(const Accum& accum, const char* prefix, bool verbose, int total = 0) const;
     void print(bool verbose) const;
 };
 
-inline void Measure::start_process() {
-    timespec n;
-    clock_gettime(CLOCK_MONOTONIC, &n);
-    if (!(t1.tv_sec == 0 and t1.tv_nsec == 0)) {
-        period.add(ts_diff(n, t1));
-    }
-    t1 = n;
-}
-
-inline void Measure::pause_process() {
-    timespec n;
-    clock_gettime(CLOCK_MONOTONIC, &n);
-    duration1.add(ts_diff(n, t1));
-}
-
-inline void Measure::cont_process() {
-    clock_gettime(CLOCK_MONOTONIC, &t2);
-}
-
-inline void Measure::stop_process() {
-    timespec n;
-    clock_gettime(CLOCK_MONOTONIC, &n);
-    duration2.add(ts_diff(n, t2));
-    duration.add(ts_diff(n, t1));
-}
-
 class MeasureThreadsafe {
  private:
-    Measure *pmeasure;
     Measure m[2];
+    Measure *pmeasure;
+    timespec t1s;
+    timespec t1e;
+    timespec t2s;
+    timespec t1old;
+    unsigned int FPUStatus;
+    unsigned int MXStatus;
     inline Measure *access() { return atomic_get(pmeasure); }
+    inline int ts_diff(const timespec& ts1, const timespec& ts2);
  public:
-    MeasureThreadsafe(): pmeasure(m) {}
-    inline void start() { access()->start_process(); }
-    inline void pause() { access()->pause_process(); }
-    inline void cont() { access()->cont_process(); }
-    inline void stop() { access()->stop_process(); }
+    MeasureThreadsafe();
+    inline void start() {
+	clear_fpu_status_bits();
+	_MM_SET_EXCEPTION_STATE(0);
+	clock_gettime(CLOCK_MONOTONIC, &t1s);
+    }
+    inline void pause() {
+	clock_gettime(CLOCK_MONOTONIC, &t1e);
+	FPUStatus = get_fpu_status_bits();
+	MXStatus = _MM_GET_EXCEPTION_STATE();
+    }
+    inline void cont() {
+	clear_fpu_status_bits();
+	_MM_SET_EXCEPTION_STATE(0);
+	clock_gettime(CLOCK_MONOTONIC, &t2s);
+    }
+    inline void stop();
     void print(bool verbose = false);
 };
 
+/* return time difference in ns, fail if > sec (doesn't fit int 32 bit int) */
+inline int MeasureThreadsafe::ts_diff(const timespec& ts1, const timespec& ts2) {
+    time_t df = ts1.tv_sec - ts2.tv_sec;
+    if (abs(df) > 2) {
+        return -1; // failed
+    }
+    return df * 1000000000 + (ts1.tv_nsec - ts2.tv_nsec);
+}
+
+
+inline void MeasureThreadsafe::stop() {
+    Measure& m = *access();
+    timespec n;
+    clock_gettime(CLOCK_MONOTONIC, &n);
+    m.FPUStatus2 |= get_fpu_status_bits();
+    m.MXStatus2 |= _MM_GET_EXCEPTION_STATE();
+    m.FPUStatus1 |= FPUStatus;
+    m.MXStatus1 |= MXStatus;
+    if (!(t1old.tv_sec == 0 && t1old.tv_nsec == 0)) {
+        m.period.add(ts_diff(t1s, t1old));
+    }
+    t1old = t1s;
+    m.duration1.add(ts_diff(t1e, t1s));
+    m.duration2.add(ts_diff(n, t2s));
+    m.duration.add(ts_diff(n, t1s));
+}
+
 extern MeasureThreadsafe measure;
 
 void add_time_measurement();
@@ -248,6 +298,9 @@ private:
     string factory_dir;
     string pixmap_dir;
     string user_dir;
+    string old_user_dir;
+    string preset_dir;
+    string pluginpreset_dir;
     string plugin_dir;
     string sys_IR_dir;
     PathList IR_pathlist;
@@ -259,6 +312,9 @@ private:
     friend CmdlineOptions& get_options();
 
 public:
+#ifndef NDEBUG
+    bool dump_parameter;
+#endif
     SkinHandling skin;
 public:
     CmdlineOptions();
@@ -269,9 +325,15 @@ public:
     string get_pixmap_filepath(const string& basename) const { return pixmap_dir + basename; }
     string get_builder_filepath(const string& basename) const { return builder_dir + basename; }
     string get_user_filepath(const string& basename) const { return user_dir + basename; }
+    string get_preset_filepath(const string& basename) const { return preset_dir + basename; }
     string get_factory_filepath(const string& basename) const { return factory_dir + basename; }
+    string get_pluginpreset_filepath(const string& id) const { return pluginpreset_dir + id; }
     const string& get_user_dir() const { return user_dir; }
+    const string& get_old_user_dir() const { return old_user_dir; }
     const string& get_plugin_dir() const { return plugin_dir; }
+    const string& get_preset_dir() const { return preset_dir; }
+    const string& get_pluginpreset_dir() const { return pluginpreset_dir; }
+    const string& get_factory_dir() const { return factory_dir; }
     const string& get_sys_IR_dir() const { return sys_IR_dir; }
     const Glib::ustring& get_rcset() const { return rcset; }
     const string& get_loadfile() const { return load_file; }
@@ -320,11 +382,13 @@ private:
     Logger();
     ~Logger();
     void write_queued();
+    friend class LoggerGuard;
 public:
     void unplug_queue();
     msg_signal& signal_message();
     void print(const char* func, const string& msg, GxMsgType msgtype);
     static Logger& get_logger();
+    static void destroy();
 };
 
 void  gx_print_logmsg(const char*, const string&, GxMsgType);
@@ -383,7 +447,6 @@ public:
  ** misc function declarations
  */
 
-bool gx_start_jack();
 int  gx_system_call(const string&, bool devnull = false, bool escape = false);
 
 template <class T>
