@@ -59,6 +59,8 @@ class GxUI {
 
  private:
     static list<GxUI*>    fGuiList;
+    static bool in_updateAll;
+    bool        in_update;
     zmap        fZoneMap;
  public:
     GxUI();
@@ -108,7 +110,7 @@ typedef GxUiItemV<bool> GxUiItemBool;
 
 template<class T>
 GxUiItemV<T>::GxUiItemV(GxUI* ui, T* zone)
-  : fGUI(ui), fZone(zone) {
+    : fGUI(ui), fZone(zone), fCache(*zone) {
     ui->registerZone(zone, this);
 }
 
