@@ -3,6 +3,9 @@
 #include <ladspa.h>
 #include <iostream>
 
+#include <glibmm/init.h>
+#include <glibmm/thread.h>
+
 #include <jack/jack.h>
 #include <jack/thread.h>
 
@@ -129,7 +132,7 @@ void ControlParameter::get_values() {
 
 class PresetIO: public gx_system::AbstractPresetIO {
 private:
-    GxJConvSettings jcset;
+    GxJConvSettings *jcset;
     ParamMap& param;
     paramlist plist;
     MidiControllerList::controller_array *midi_list;
@@ -148,7 +151,7 @@ public:
 
 // cp not yet initialized, only use address!
 PresetIO::PresetIO(ParamMap& param_, ConvolverAdapter* convolver_, ControlParameter& cp)
-    : jcset(),
+    : jcset(0),
       param(param_),
       plist(),
       midi_list(0),
@@ -164,6 +167,8 @@ void PresetIO::clear() {
     plist.clear();
     delete midi_list;
     midi_list = 0;
+    delete jcset;
+    jcset = 0;
 }
 
 void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader& head) {
@@ -192,10 +197,10 @@ void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
         } else if (jp.current_value() == "jconv") {
 	    if (convolver) {
 		gx_system::PathList sl;
-		sl.add(string(getenv("HOME"))+"/.gx_head"); //FIXME
+		sl.add(Glib::build_filename(Glib::get_user_config_dir(), "guitarix")); //FIXME
 		sl.add(GX_SOUND_DIR);
-		jcset = GxJConvSettings();
-		jcset.readJSON(jp, sl);
+		jcset = new GxJConvSettings();
+		jcset->readJSON(jp, sl);
 	    } else {
 		jp.skip_object();
 	    }
@@ -212,8 +217,8 @@ void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 }
 
 void PresetIO::commit_preset() {
-    if (convolver) {
-	convolver->jcset = jcset;
+    if (convolver && jcset) {
+	convolver->set(*jcset);
     }
     for (gx_engine::paramlist::iterator i = plist.begin(); i != plist.end(); ++i) {
         (*i)->setJSON_value();
@@ -282,19 +287,79 @@ void StateIO::write_state(gx_system::JsonWriter &jw, bool preserve_preset) {}
 
 class LadspaSettings: public gx_system::GxSettingsBase {
 private:
+    enum Source {
+	state,
+	preset,
+    };
+    Source current_source;
     PresetIO preset_io;
     StateIO state_io;
+    void change_preset_file(const std::string& newfile);
+    void load(Source src, const Glib::ustring& name);
 public:
+    gx_system::PresetFile presetfile;
     LadspaSettings(string sfname, string presname,
 		   ParamMap& param, EngineControl&, ConvolverAdapter*, ControlParameter&);
     ~LadspaSettings();
     void load(int num);
+    bool idx_in_preset(int idx) { return idx >= 0 && idx < presetfile.size(); }
+    void load_preset_by_idx(int idx) { load(preset, presetfile.get_name(idx)); }
 };
 
+void LadspaSettings::load(Source src, const Glib::ustring& name) {
+    gx_system::PresetFile *p = 0;
+    switch (src) {
+    case preset:
+	if (presetfile.get_index(name) < 0) {
+	    return;
+	}
+	p = &presetfile;
+	current_source = src;
+	current_name = name;
+	current_bank = "";
+	break;
+    case state:
+    default:
+	current_source = state;
+	current_bank = current_name = "";
+	break;
+    }
+    seq.start_ramp_down();
+    loadsetting(p, name);
+    seq.start_ramp_up();
+    if (current_source == state) {
+	// might have changed because we read all state file sections
+	current_bank = current_name = "";
+    }
+    gx_ui::GxUI::updateAllGuis();
+    seq.clear_rack_changed();
+    selection_changed();
+}
+
+void LadspaSettings::change_preset_file(const std::string& newfile) {
+    try {
+	if (presetfile.get_filename() == newfile) {
+	    presetfile.reopen();
+	} else {
+	    presetfile.open(newfile);
+	    if (current_source == preset) {
+		current_source = state;
+		current_name = "";
+		selection_changed();
+	    }
+	}
+	presetlist_changed();
+    } catch(gx_system::JsonException& e) {
+	gx_system::gx_print_error(
+	    newfile.c_str(), e.what());
+    }
+}
+
 // seq and cp not yet initialized, only use address!
 LadspaSettings::LadspaSettings(string sfname, string presname, ParamMap& param,
 			       EngineControl& seq, ConvolverAdapter* convolver, ControlParameter& cp)
     : GxSettingsBase(seq),
+      current_source(state),
       preset_io(param, convolver, cp),
       state_io(param, convolver, cp) {
     set_io(&state_io, &preset_io);
@@ -308,12 +373,14 @@ LadspaSettings::~LadspaSettings() {
 void LadspaSettings::load(int num) {
     if (num == 0) {
 	statefile.ensure_is_current();
-	gx_system::GxSettingsBase::load(state,"","");
-    } else if (idx_in_preset(num-1)) {
-	presetfile.ensure_is_current();
-	load_preset_by_idx(num-1);
+	load(state,"");
     } else {
-	gx_system::gx_print_error("preset loader", boost::format("no preset number %1%") % num);
+	presetfile.ensure_is_current();
+	if (idx_in_preset(num-1)) {
+	    load_preset_by_idx(num-1);
+	} else {
+	    gx_system::gx_print_error("preset loader", boost::format("no preset number %1%") % num);
+	}
     }
 }
 
@@ -335,6 +402,27 @@ static void log_terminal(const string& msg, gx_system::GxMsgType tp, bool plugge
     }
 }
 
+static int logger_inited = 0;
+
+static void init_logger() {
+    if (logger_inited++) {
+	return;
+    }
+    gx_system::Logger& log(gx_system::Logger::get_logger());
+    if (log.signal_message().empty()) {
+	log.signal_message().connect(sigc::ptr_fun(log_terminal));
+	log.unplug_queue();
+    }
+}
+
+static void destroy_logger() {
+    if (--logger_inited > 0) {
+	return;
+    }
+    gx_system::Logger::destroy();
+}
+
+
 /****************************************************************
  ** class LadspaGuitarix
  */
@@ -357,6 +445,7 @@ private:
 	PresetLoader();
 	~PresetLoader();
     public:
+	static void start_presetloader() { if (!instance) create(); }
 	static void create();
 	static void destroy();
 	static void add_instance(LadspaGuitarix* i);
@@ -388,10 +477,11 @@ protected:
     LadspaGuitarix(EngineControl& engine, ConvolverAdapter* convolver,
 		   ControlParameter& cp, const char *envvar);
     ~LadspaGuitarix();
+    static void start_presetloader() { PresetLoader::start_presetloader(); }
 };
 
 static string get_statefile() {
-    return string(getenv("HOME")) + "/.gx_head/gx_head_rc";
+    return Glib::build_filename(Glib::get_user_config_dir(), "guitarix/gx_head_rc");
 }
 
 static string get_presetfile(const char *envvar) {
@@ -403,7 +493,7 @@ static string get_presetfile(const char *envvar) {
     if (path && path[0]) {
 	return path;
     }
-    return string(getenv("HOME")) + "/.gx_head/gx_headpre_rc";
+    return Glib::build_filename(Glib::get_user_config_dir(), "guitarix/banks/ladspa.gx");
 }
 
 // engine and cp not yet initialized, only use address!
@@ -469,7 +559,7 @@ void LadspaGuitarix::prepare_run() {
     pthread_t tid = pthread_self();
     if (!pthread_equal(tid, last_thread_id)) {
 	last_thread_id = tid;
-	AVOIDDENORMALS;
+	AVOIDDENORMALS();
     }
     check_preset();
     control_parameter.get_values();
@@ -554,12 +644,14 @@ void LadspaGuitarix::PresetLoader::destroy() {
 }
 
 void LadspaGuitarix::PresetLoader::run_mainloop() {
+    init_logger();
     instance = new PresetLoader();
     instance->new_preset.connect(sigc::mem_fun(*instance, &PresetLoader::load_presets));
     sem_post(&instance->created_sem);
     instance->mainloop->run();
     delete instance;
     instance = 0;
+    destroy_logger();
 }
 
 void LadspaGuitarix::PresetLoader::load_presets() {
@@ -778,8 +870,8 @@ MonoEngine::MonoEngine(const string& plugin_dir, ParamMap& param, ParameterGroup
 	  "tube.select", _("select"), ampstack_groups),
       // internal audio modules
       noisegate(),
-      cabinet(*this, resamp),
-      contrast(*this, resamp) {
+      cabinet(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
+      contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp) {
 
     cabinet.set_sync(true);
     contrast.set_sync(true);
@@ -864,6 +956,8 @@ void MonoEngine::load_static_plugins() {
     pl.add(gx_effects::gx_feedback::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&tonestack.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&cabinet.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::abgate::plugin(),           PLUGIN_POS_RACK);
+    pl.add(pluginlib::vibe::plugin_mono(),        PLUGIN_POS_RACK);
 }
 
 
@@ -949,7 +1043,7 @@ public:
 
 LadspaGuitarixMono::LadspaGuitarixMono(unsigned long sr)
     : LadspaGuitarix(engine, 0, control_parameter, "LADSPA_GUITARIX_MONO_PRESET"),
-      engine(string(getenv("HOME"))+"/.gx_head/", param, get_group_table()),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), param, get_group_table()),
       control_parameter(GUITARIX_PARAM_COUNT),
       rebuffer(),
       volume_port(),
@@ -1043,6 +1137,7 @@ void LadspaGuitarixMono::connectPortToGuitarix(
 
 LADSPA_Handle LadspaGuitarixMono::instantiateGuitarix(
     const LADSPA_Descriptor * Descriptor, unsigned long SampleRate) {
+    start_presetloader();
     return new LadspaGuitarixMono(SampleRate);
 }
 
@@ -1236,7 +1331,7 @@ public:
     StereoModuleChain stereo_chain;
     ConvolverAdapter convolver;
 public:
-    StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups);
+    StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::PathList& pathlist);
     ~StereoEngine();
     virtual void wait_ramp_down_finished();
     virtual bool update_module_lists();
@@ -1300,10 +1395,10 @@ void StereoEngine::set_samplerate(unsigned int samplerate) {
     EngineControl::set_samplerate(samplerate);
 }
 
-StereoEngine::StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups)
+StereoEngine::StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::PathList& pathlist)
     : EngineControl(),
       // internal audio modules
-      convolver(*this) {
+      convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync), param, pathlist, "") {
 
     convolver.set_sync(true);
 
@@ -1350,6 +1445,7 @@ void StereoEngine::load_static_plugins() {
     pl.add(&convolver.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::stereoverb::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::zita_rev1::plugin(),        PLUGIN_POS_RACK);
+    pl.add(pluginlib::vibe::plugin_stereo(),      PLUGIN_POS_RACK);
 }
 
 
@@ -1357,6 +1453,14 @@ void StereoEngine::load_static_plugins() {
  ** class LadspaGuitarixStereo
  */
 
+class LadspaPathList: public gx_system::PathList {
+public:
+    inline LadspaPathList() {
+	add(Glib::build_filename(Glib::get_user_config_dir(), "guitarix")); //FIXME
+	add(GX_SOUND_DIR);
+    };
+};
+
 class LadspaGuitarixStereo: LadspaGuitarix {
 private:
 
@@ -1424,6 +1528,7 @@ private:
 		 LADSPA_Data *output_buffer1, LADSPA_Data *output_buffer2);
     };
 
+    LadspaPathList pathlist;
     StereoEngine engine;
     ControlParameter control_parameter;
     ReBuffer rebuffer;
@@ -1448,7 +1553,8 @@ public:
 
 LadspaGuitarixStereo::LadspaGuitarixStereo(unsigned long sr)
     : LadspaGuitarix(engine, &engine.convolver, control_parameter, "LADSPA_GUITARIX_STEREO_PRESET"),
-      engine(string(getenv("HOME"))+"/.gx_head/", param, get_group_table()),
+      pathlist(),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), param, get_group_table(), pathlist),
       control_parameter(GUITARIX_PARAM_COUNT),
       rebuffer(),
       volume_port(),
@@ -1552,6 +1658,7 @@ void LadspaGuitarixStereo::connectPortToGuitarix(
 
 LADSPA_Handle LadspaGuitarixStereo::instantiateGuitarix(
     const LADSPA_Descriptor * Descriptor, unsigned long SampleRate) {
+    start_presetloader();
     return new LadspaGuitarixStereo(SampleRate);
 }
 
@@ -1778,11 +1885,6 @@ const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index) {
 	inited = 1;
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-	gx_system::Logger& log(gx_system::Logger::get_logger());
-	if (log.signal_message().empty()) {
-	    log.signal_message().connect(sigc::ptr_fun(log_terminal));
-	    log.unplug_queue();
-	}
     }
     switch (Index) {
     case 0:
